<?php

declare(strict_types=1);

namespace Drupal\markdown_importer\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Batch\BatchBuilder;
use Drupal\markdown_importer\Service\MarkdownProcessor;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form for importing Markdown files from GitHub and GitLab repositories.
 */
class MarkdownImporterForm extends FormBase {

  /**
   * The Markdown processor service.
   *
   * @var \Drupal\markdown_importer\Service\MarkdownProcessor
   */
  protected MarkdownProcessor $markdownProcessor;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs a MarkdownImportForm object.
   *
   * @param \Drupal\markdown_importer\Service\MarkdownProcessor $markdownProcessor
   *   The Markdown processor service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    MarkdownProcessor $markdownProcessor,
    EntityTypeManagerInterface $entityTypeManager,
  ) {
    $this->markdownProcessor = $markdownProcessor;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('markdown_importer.processor'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'markdown_import_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $content_types = $this->getContentTypes();

    $form['repo_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Repository URL'),
      '#description' => $this->t('Enter the URL of the GitHub or GitLab repository containing Markdown files.'),
      '#required' => TRUE,
    ];

    $form['platform'] = [
      '#type' => 'select',
      '#title' => $this->t('Platform'),
      '#options' => [
        'github' => $this->t('GitHub'),
        'gitlab' => $this->t('GitLab'),
        'drupal' => $this->t('Drupal'),
      ],
      '#required' => TRUE,
    ];

    $form['content_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Content Type'),
      '#options' => $content_types,
      '#description' => $this->t('Select the content type to use for imported Markdown files.'),
      '#required' => TRUE,
      '#ajax' => [
        'callback' => '::updateTargetField',
        'wrapper' => 'target-field-wrapper',
      ],
    ];

    $form['target_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Target Field'),
      '#options' => $form_state->getValue('content_type') ? $this->getContentTypeFields($form_state->getValue('content_type')) : [],
      '#description' => $this->t('Select the field to store the Markdown content.'),
      '#required' => TRUE,
      '#prefix' => '<div id="target-field-wrapper">',
      '#suffix' => '</div>',
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Import Markdown'),
    ];

    return $form;
  }

  /**
   * AJAX callback to update the target field options based on content type.
   */
  public function updateTargetField(array &$form, FormStateInterface $form_state): array {
    return $form['target_field'];
  }

  /**
   * Fetches all available content types.
   *
   * @return array
   *   An array of content type machine names and labels.
   */
  protected function getContentTypes(): array {
    $content_types = $this->entityTypeManager->getStorage('node_type')
      ->loadMultiple();
    $options = [];
    foreach ($content_types as $content_type) {
      $options[$content_type->id()] = $content_type->label();
    }
    return $options;
  }

  /**
   * Fetches fields for a given content type.
   *
   * @param string $content_type
   *   The machine name of the content type.
   *
   * @return array
   *   An array of field machine names and labels.
   */
  protected function getContentTypeFields(string $content_type): array {
    $fields = $this->entityTypeManager->getStorage('field_config')
      ->loadByProperties(['entity_type' => 'node', 'bundle' => $content_type]);
    $options = [];
    foreach ($fields as $field) {
      if ($field->getType() === 'text_long' ||
        $field->getType() === 'text_with_summary') {
        $options[$field->getName()] = $field->label();
      }
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $repo_url = $form_state->getValue('repo_url');
    $platform = $form_state->getValue('platform');
    $content_type = $form_state->getValue('content_type');
    $target_field = $form_state->getValue('target_field');

    // Fetch the file metadata for the repository.
    $files = $this->markdownProcessor->fetchMarkdownFiles($repo_url, $platform);

    $batch_builder = new BatchBuilder();
    $batch_builder
      ->setTitle($this->t('Importing Markdown files...'))
      ->setInitMessage($this->t('Starting the import process...'))
      ->setProgressMessage($this->t('Processing files...'));

    // Add a separate batch operation for each file.
    foreach ($files as $file) {
      $batch_builder->addOperation([
        $this->markdownProcessor, 'processFileBatchOperation',
      ], [$file['download_url'], $file['name'], $content_type, $target_field]);
    }

    $batch_builder->setFinishCallback([$this, 'finishCallback']);

    batch_set($batch_builder->toArray());
  }

  /**
   * Batch finish callback.
   *
   * @param bool $success
   *   Whether the batch process completed successfully.
   * @param array $results
   *   The results of the batch process.
   * @param array $operations
   *   The operations that were performed.
   */
  public function finishCallback(bool $success, array $results, array $operations): void {
    if ($success) {
      $processed_count = count($results['processed'] ?? []);
      $this->messenger()->addStatus($this->t('Successfully imported @count Markdown files.', ['@count' => $processed_count]));
    }
    else {
      $this->messenger()->addError($this->t('There were errors during the import process.'));
    }
  }

}
