<?php

declare(strict_types=1);

namespace Drupal\markdown_importer\Plugin\Filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\filter\Attribute\Filter;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Drupal\filter\Plugin\FilterInterface;
use Drupal\markdown_importer\Service\MarkdownProcessorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a filter to format markdown.
 */
#[Filter(
  id: "markdown_filter",
  title: new TranslatableMarkup("Markdown Filter"),
  description: new TranslatableMarkup("A Markdown filter to transform markdown into HTML."),
  type: FilterInterface::TYPE_TRANSFORM_REVERSIBLE,
)]
class MarkdownFilter extends FilterBase implements ContainerFactoryPluginInterface {


  /**
   * Markdown processor.
   *
   * @var \Drupal\markdown_importer\Service\MarkdownProcessorInterface
   */
  protected MarkdownProcessorInterface $markdownProcessor;

  /**
   * Constructs a FormatterBase object.
   *
   * @param array $configuration
   *   The configuration of the filter.
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\markdown_importer\Service\MarkdownProcessorInterface $markdown
   *   Markdown processor.
   */
  final public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    MarkdownProcessorInterface $markdown,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->markdownProcessor = $markdown;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('markdown_importer.processor')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {

    $form['tips'] = [
      '#type' => 'item',
      '#title' => $this->t('Important'),
      '#description' => $this->t('<ul><li>The Markdown filter should run before the "Convert line breaks into HTML(i . e . < br > and < p > )" filter.)</li><li>The Markdown filter should run before the "Limit allowed HTML tags and correct faulty HTML" filter. It is strongly recommended to use these filters together.</li></ul>'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode): FilterProcessResult {
    $converted = $this->markdownProcessor->formatMarkdown($text);
    return new FilterProcessResult($converted);
  }

  /**
   * {@inheritdoc}
   */
  public function tips($long = FALSE): string {
    return (string) $this->t('Parses markdown and converts it to HTML.');
  }

}
