<?php

declare(strict_types=1);

namespace Drupal\markdown_importer\Service;

use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use GuzzleHttp\ClientInterface;
use League\CommonMark\Environment\Environment;
use League\CommonMark\Extension\Attributes\AttributesExtension;
use League\CommonMark\Extension\CommonMark\CommonMarkCoreExtension;
use League\CommonMark\Extension\HeadingPermalink\HeadingPermalinkExtension;
use League\CommonMark\GithubFlavoredMarkdownConverter;

/**
 * Service for processing and importing Markdown files.
 */
class MarkdownProcessor implements MarkdownProcessorInterface {

  use DependencySerializationTrait;
  use LoggerChannelTrait;

  /**
   * Module handler.
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The entity type manager.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The HTTP client.
   */
  protected ClientInterface $httpClient;

  /**
   * Constructs a MarkdownProcessor object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \GuzzleHttp\ClientInterface $httpClient
   *   The HTTP client.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    ClientInterface $httpClient,
    ModuleHandlerInterface $module_handler
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->httpClient = $httpClient;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public function fetchMarkdownFiles(string $repo_url, string $platform): array {
    $markdown_files = [];

    // Extract the project path from the repository URL.
    $project_path = $this->extractProjectPath($repo_url, $platform);
    // Recursive function to fetch contents of directories.
    $fetchContents = function ($api_url) use (&$fetchContents, &$markdown_files, $platform, $project_path) {
      try {
        $response = $this->httpClient->get($api_url, [
          'headers' => ['Accept' => 'application/json'],
        ]);

        $items = json_decode($response->getBody()->getContents(), TRUE);

        // Check if $items is a valid array before processing.
        if (!is_array($items)) {
          $this->getLogger('markdown_importer')
            ->error('Invalid response from API: @api_url. res: @response', [
              '@api_url' => $api_url,
              '@response' => $response->getBody()->getContents(),
            ]);
          return [];
        }

        foreach ($items as $item) {
          // If it's a directory, fetch its contents recursively.
          if (($platform === 'github' && $item['type'] === 'dir') ||
            ($platform === 'gitlab' && $item['type'] === 'tree') ||
            ($platform === 'drupal' && $item['type'] === 'tree')) {
            $sub_api_url = ($platform === 'github')
              ? $item['url']
              : (($platform === 'gitlab')
                ? $this->buildGitLabTreeApiUrl($item['path'], $project_path, $platform)
                : $this->buildGitLabTreeApiUrl($item['path'], $project_path, $platform));
            $fetchContents($sub_api_url);
          }
          // If it's a Markdown file, add it to the results.
          elseif (pathinfo($item['name'], PATHINFO_EXTENSION) === 'md') {
            $markdown_files[] = [
              'name' => $item['name'],
              'download_url' => ($platform === 'github')
                ? $item['download_url']
                : (($platform === 'gitlab')
                  ? $this->buildGitLabRawUrl($item['path'], $project_path, $platform)
                  : $this->buildGitLabRawUrl($item['path'], $project_path, $platform)),
            ];
          }
        }
      }
      catch (\Exception $e) {
        $this->getLogger('markdown_importer')->error('Error fetching files: @message', [
          '@message' => $e->getMessage(),
        ]);
      }
    };

    // Start with the root directory.
    $api_url = $this->buildApiUrl($repo_url, $platform);
    $fetchContents($api_url);

    return $markdown_files;
  }

  /**
   * {@inheritdoc}
   */
  public function extractProjectPath(string $repo_url, string $platform): string {
    if ($platform === 'github') {
      // For GitHub, the project path is in the format "owner/repo".
      if (preg_match('/github.com\/([^\/]+)\/([^\/]+)/', $repo_url, $matches)) {
        return "{$matches[1]}/{$matches[2]}";
      }
      throw new \InvalidArgumentException('Invalid GitHub repository URL.');
    }

    if ($platform === 'gitlab') {
      // For GitLab, the project path may include subgroups.
      if (preg_match('/gitlab.com\/((?:[^\/]+\/)+[^\/]+)/', $repo_url, $matches)) {
        return $matches[1];
      }
      throw new \InvaliArgumentException('Invalid GitLab repository URL.');
    }

    if ($platform === 'drupal') {
      // For Drupal, the project path is in the format "project/repo".
      if (preg_match('/git.drupalcode.org\/project\/([^\/]+)/', $repo_url, $matches)) {
        return "project/{$matches[1]}";
      }
      throw new \InvalidArgumentException('Invalid Drupal repository URL.');
    }

    throw new \InvalidArgumentException('Unsupported platform.');
  }

  /**
   * {@inheritdoc}
   */
  public function processFileBatchOperation(string $file_url, string $file_name, string $content_type, string $target_field, array &$context): void {
    try {
      // Fetch raw Markdown content.
      $response = $this->httpClient->get($file_url);
      $markdown_content = $response->getBody()->getContents();
      // Convert Markdown content to HTML.
      $html_content = $this->formatMarkdown($markdown_content);

      // Extract the title from the Markdown content.
      $title = $this->extractTitle($markdown_content) ?: pathinfo($file_name, PATHINFO_FILENAME);

      $node_storage = $this->entityTypeManager->getStorage('node');

      // Check if the field_git_repository_source_file exists for this type.
      $fields = $this->entityTypeManager->getStorage('field_storage_config')
        ->loadByProperties([
          'entity_type' => 'node',
          'field_name' => 'field_git_repository_source_file',
        ]);

      $has_repository_field = !empty($fields);

      // Check for existing nodes based on repository field or title.
      if ($has_repository_field) {
        $existing_nodes = $node_storage->loadByProperties([
          'type' => $content_type,
          'field_git_repository_source_file' => $file_url,
        ]);
      }
      else {
        $existing_nodes = $node_storage->loadByProperties([
          'type' => $content_type,
          'title' => $title,
        ]);
      }

      if ($existing_nodes) {
        // Update existing node.
        $node = reset($existing_nodes);
        $node->set('title', $title);
        $node->set($target_field, [
          'value' => $html_content,
        ]);

        if ($has_repository_field) {
          $node->set('field_git_repository_source_file', $file_url);
        }
      }
      else {
        // Create a new node.
        $node_data = [
          'type' => $content_type,
          'title' => $title,
          $target_field => [
            'value' => $html_content,
          ],
          'status' => 1,
        ];

        if ($has_repository_field) {
          $node_data['field_git_repository_source_file'] = $file_url;
        }

        $node = $node_storage->create($node_data);
      }

      $node->save();
      $context['results']['processed'][] = $file_url;
    }
    catch (\Exception $e) {
      $this->getLogger('markdown_importer')->error('Error processing file @file: @message', [
        '@file' => $file_name,
        '@message' => $e->getMessage(),
      ]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildGitLabTreeApiUrl(string $path, string $project_path, string $platform): string {
    $encoded_project_path = ($platform === 'drupal')
      ? urlencode("project/{$project_path}")
      : urlencode($project_path);
    $encoded_path = urlencode($path);

    return ($platform === 'drupal')
      ? "https://git.drupalcode.org/api/v4/projects/{$encoded_project_path}/repository/tree?path={$encoded_path}"
      : "https://gitlab.com/api/v4/projects/{$encoded_project_path}/repository/tree?path={$encoded_path}";
  }

  /**
   * {@inheritdoc}
   */
  public function buildGitLabRawUrl(string $path, string $project_path, string $platform): string {
    $encoded_project_path = urlencode($project_path);
    $encoded_file_path = urlencode($path);

    $base_url = ($platform === 'drupal')
        ? "https://git.drupalcode.org/api/v4/projects"
        : "https://gitlab.com/api/v4/projects";

    return "{$base_url}/{$encoded_project_path}/repository/files/{$encoded_file_path}/raw";
  }

  /**
   * {@inheritdoc}
   */
  public function buildApiUrl(string $repo_url, string $platform): string {
    if ($platform === 'github') {
      if (preg_match('/github\.com\/([^\/]+)\/([^\/]+)/', $repo_url, $matches)) {
        return "https://api.github.com/repos/{$matches[1]}/{$matches[2]}/contents";
      }
      throw new \InvalidArgumentException('Invalid GitHub repository URL.');
    }
    elseif ($platform === 'gitlab') {
      if (preg_match('/gitlab\.com\/((?:[^\/]+\/)+[^\/]+)/', $repo_url, $matches)) {
        $encoded_project_path = urlencode($matches[1]);
        return "https://gitlab.com/api/v4/projects/{$encoded_project_path}/repository/tree";
      }
      throw new \InvalidArgumentException('Invalid GitLab repository URL.');
    }
    elseif ($platform === 'drupal') {
      if (preg_match('/git\.drupalcode\.org\/project\/([^\/]+)/', $repo_url, $matches)) {
        $project_path = $matches[1];
        $encoded_project_path = urlencode("project/{$project_path}");
        return "https://git.drupalcode.org/api/v4/projects/{$encoded_project_path}/repository/tree";
      }
      throw new \InvalidArgumentException('Invalid Drupal repository URL.');
    }

    throw new \InvalidArgumentException('Unsupported platform or invalid repository URL.');
  }

  /**
   * {@inheritdoc}
   */
  public function extractTitle(string $markdown_content): ?string {
    if (preg_match('/^# (.+)/m', $markdown_content, $matches)) {
      return $matches[1];
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function formatMarkdown(string $markdown_content): string {
    // Create the environment with the necessary extensions.
    $environment = new Environment([
      'html_input' => 'allow',
      'allow_unsafe_links' => FALSE,
      'heading_permalink' => [
        'html_class' => 'anchor',
        'id_prefix' => '',
        'insert' => 'before',
        'symbol' => '#',
        'aria_hidden' => TRUE,
      ],
      'attributes' => [
        'allow_attributes' => TRUE,
      ],
    ]);

    // Add extensions.
    $environment->addExtension(new CommonMarkCoreExtension());
    $environment->addExtension(new HeadingPermalinkExtension());
    $environment->addExtension(new AttributesExtension());

    // Initialize the converter with GitHub-flavored Markdown support.
    $converter = new GithubFlavoredMarkdownConverter([], $environment);

    // Convert the Markdown content to HTML.
    return (string) $converter->convert($markdown_content);
  }

}
