<?php

declare(strict_types=1);

namespace Drupal\markdown_importer\Service;

/**
 * Interface for processing and importing Markdown files.
 */
interface MarkdownProcessorInterface {

  /**
   * Processes a single Markdown file and creates or updates a node.
   *
   * @param string $file_url
   *   The file's download URL.
   * @param string $file_name
   *   The file name.
   * @param string $content_type
   *   Content type.
   * @param string $target_field
   *   Target field.
   * @param array $context
   *   The batch context.
   */
  public function processFileBatchOperation(string $file_url, string $file_name, string $content_type, string $target_field, array &$context): void;

  /**
   * Fetches Markdown files from the repository.
   *
   * @param string $repo_url
   *   The repository URL.
   * @param string $platform
   *   The platform (GitHub or GitLab).
   *
   * @return array
   *   An array of Markdown file metadata.
   */
  public function fetchMarkdownFiles(string $repo_url, string $platform): array;

  /**
   * Builds the API URL for the given repository and platform.
   *
   * @param string $repo_url
   *   The repository URL.
   * @param string $platform
   *   The platform (GitHub or GitLab or self-hosted).
   *
   * @return string
   *   The API URL.
   */
  public function buildApiUrl(string $repo_url, string $platform): string;

  /**
   * Extracts the title from the Markdown content.
   *
   * @param string $markdown_content
   *   The raw Markdown content.
   *
   * @return string
   *   The extracted title or a default title if none is found.
   */
  public function extractTitle(string $markdown_content): ?string;

  /**
   * Builds the raw URL for a GitLab or Drupal file.
   *
   * @param string $path
   *   The file path within the repository.
   * @param string $project_path
   *   The encoded project path.
   * @param string $platform
   *   The platform ('gitlab' or 'drupal').
   *
   * @return string
   *   The raw file URL.
   */
  public function buildGitLabRawUrl(string $path, string $project_path, string $platform): string;

  /**
   * Builds the GitLab tree API URL.
   *
   * @param string $path
   *   The path in the repository.
   * @param string $project_path
   *   The project path.
   * @param string $platform
   *   The platform (GitLab or Drupal).
   *
   * @return string
   *   The constructed API URL.
   */
  public function buildGitLabTreeApiUrl(string $path, string $project_path, string $platform): string;

  /**
   * Extracts the project path from the repository URL.
   *
   * @param string $repo_url
   *   The repository URL.
   * @param string $platform
   *   The platform (GitHub or GitLab or Drupal).
   *
   * @return string
   *   The extracted project path.
   */
  public function extractProjectPath(string $repo_url, string $platform): string;

  /**
   * Format markdown content.
   *
   * @param string $markdown_content
   *   Markdown content.
   *
   * @return string
   *   Formatted content.
   */
  public function formatMarkdown(string $markdown_content): string;

}
